App = Ember.Application.create();

App.Router.map(function() {
  this.route('about');
  this.resource('users', function() {
    this.route('new');
    this.resource('user', { path: ':user_id' }, function() {
      this.route('edit');
    });
  })
});

App.UsersRoute = Ember.Route.extend({
  model: function() {
    return App.User.find();
  }
});

App.UserEditRoute = Ember.Route.extend({
  model: function() {
    return this.modelFor('user');
  },

  renderTemplate: function() {
    this.render({ into: 'users' });
  },

  events: {
    save: function(model) {
      model.save().then( function() {
        App.Router.router.transitionTo('users.index');
      }, function() {
        alert('save failed!');
      });
    },
    undo: function(model) {
      model.rollback();
      App.Router.router.transitionTo('users.index');
    }
  }
});

App.UsersNewRoute = Ember.Route.extend({
  model: function() {
    return App.User.createRecord();
  },

  renderTemplate: function() {
    this.render({ into: 'users' });
  },

  activate: function() {
    this.controllerFor('users').set('newUserCreate', true);
  },

  deactivate: function() {
    this.controllerFor('users').set('newUserCreate', false);
    if (this.currentModel.get('isNew') && this.currentModel.get('isSaving') == false) {
      this.currentModel.get('transaction').rollback();
    } 
  }
});

App.UsersController = Ember.ArrayController.extend({
  sortProperties: ['lastName'],
  sortAscending: true
});

App.UserController = Ember.ObjectController.extend({
  gravatarUrl: function() {
    var email = this.get('email');
    var hash = CryptoJS.MD5(email.toLowerCase());
    var md5_sum = hash.toString(CryptoJS.enc.Hex);

   return 'http://www.gravatar.com/avatar/' + md5_sum;
  }.property('email'),

  syntacticallyPlausibleEmail: function() { 
    var email = this.get('email');
    var re = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return re.test(email);
  }.property('email')
});

App.UsersNewController = Ember.ObjectController.extend({
  save: function(model) {
    model.get('transaction').commit();
    App.Router.router.transitionTo('users.index');
  },
  discard: function(model) {
    model.get('transaction').rollback();
    App.Router.router.transitionTo('users.index');
  }
});

App.Store = DS.Store.extend({
  adapter: 'DS.FixtureAdapter'
});

App.User = DS.Model.extend({
  firstName: DS.attr('string'),
  lastName: DS.attr('string'),
  email: DS.attr('sting')
});

App.User.FIXTURES = [{
  id: 1,
  firstName: "Linus",
  lastName: "Torvalds",
  email: "torvalds@linux-foundation.org"
}, {
  id: 2,
  firstName: "John",
  lastName: "Hall",
  email: "maddog@li.org"
}, {
  id: 3,
  firstName: "Yehuda",
  lastName: "Katz",
  email: "wycats@tilde.io"
}, {
  id: 4,
  firstName: "Tom",
  lastName: "Dale",
  email: "tom@tilde.io"
}]